@@ -1037,6 +1037,16 @@ class ExtensionDriver(object):
         """
         pass
 
+    @property
+    def extension_aliases(self):
+        """List of extension aliases supported by the driver.
+
+        Return a list of aliases identifying the core API extensions
+        supported by the driver. By default this just returns the
+        extension_alias property for backwards compatibility.
+        """
+        return [self.extension_alias]
+
     def process_create_network(self, plugin_context, data, result):
         """Process extended attributes for create network.
 
